import os
import unittest

from esse import ESSE
from esse.settings import SCHEMAS_DIR, EXAMPLES_DIR
from esse.utils import parseIncludeReferenceStatements

esse = ESSE()


class TestCase(unittest.TestCase):
    def __init__(self, example_path):
        super(TestCase, self).__init__(methodName="runTest")
        self.example_path = example_path
        self.schema_path = self.example_path.replace(EXAMPLES_DIR, SCHEMAS_DIR)

    def runTest(self):
        schema = parseIncludeReferenceStatements(self.schema_path)
        example = parseIncludeReferenceStatements(self.example_path)
        esse.validate(example, schema)


class TestResult(unittest.TextTestResult):
    def getDescription(self, test):
        return test.example_path.replace("".join((EXAMPLES_DIR, "/")), "")


def validate_examples():
    """
    Validates examples against their schemas.
    """
    suite = unittest.TestSuite()
    for root, dirs, files in os.walk(EXAMPLES_DIR):
        for file_ in files:
            TestCase(example_path=os.path.join(root, file_)).run()
            suite.addTest(TestCase(example_path=os.path.join(root, file_)))
    unittest.TextTestRunner(verbosity=2, resultclass=TestResult).run(suite)


if __name__ == "__main__":
    validate_examples()



import os
import json
import json_include

from slugify import slugify
from collections import OrderedDict

from esse.settings import SCHEMAS_DIR, EXAMPLES_DIR


def parseIncludeReferenceStatements(file_path):
    """
    Resolves `include` and `$ref` statements.

    Args:
        file_path (str): file to parse.

    Returns:
         dict|list
    """
    dirName = os.path.dirname(file_path)
    baseName = os.path.basename(file_path)
    return json.loads(json_include.build_json(dirName, baseName))


def parseIncludeReferenceStatementsByDir(dir_path):
    """
    Resolves `include` and `$ref` statements for all the JSON files inside a given directory.

    Args:
        dir_path (str): directory to parse.

    Returns:
         dict|list
    """
    data = []
    for root, dirs, files in os.walk(dir_path):
        for file_ in files:
            if os.path.splitext(file_)[1] != ".json":
                continue
            file_path = os.path.join(root, file_)
            data.append(parseIncludeReferenceStatements(file_path))
    return data


def read_json_file(path_):
    """
    Reads the given JSON file into an ordered dictionary.

    Args:
        path_ (str): path to JSON file.

    Returns:
        OrderedDict
    """
    with open(path_, "r") as f:
        return json.loads(f.read(), object_pairs_hook=OrderedDict)


def dump_json_file(path_, content, sort_keys=True):
    """
    Dumps a given JSON content. The keys are sorted if `sort_keys` is set.

    Args:
        path_ (str): path to JSON file.
        content (dict): JSON file content.
        sort_keys (bool): whether to sort keys. Defaults to True.
    """
    with open(path_, "w+") as f:
        f.write("".join((json.dumps(content, separators=(",", ": "), indent=4, sort_keys=sort_keys), "\n")))


def refactor_examples():
    """
    Refactors examples:
        - fix space and indentation
        - sort keys
    """
    for root, dirs, files in os.walk(EXAMPLES_DIR):
        for file_ in files:
            path_ = os.path.join(root, file_)
            content = read_json_file(path_)
            dump_json_file(path_, content, True)


def refactor_schemas():
    """
    Refactors schemas:
        - fix space and indentation
        - preserve keys order
    """
    for root, dirs, files in os.walk(SCHEMAS_DIR):
        for file_ in files:
            set_schema_id(os.path.join(root, file_))


def set_schema_id(path_):
    """
    Slugifies the schema path and uses it as schema ID.

    Args:
        path_ (str): path to JSON file.
    """
    content = read_json_file(path_)
    if not content.get("$schema"):
        return  # do not add ID to non-schema files
    if content.get("$id"):
        del content["$id"]
    schema_id = slugify(path_.replace("{}/".format(SCHEMAS_DIR), "").replace(".json", ""))
    content = OrderedDict(list(OrderedDict({"schemaId": schema_id}).items()) + list(content.items()))
    dump_json_file(path_, content, False)



import os
from setuptools import find_packages, setup


def get_files_by_path(path):
    files = []
    for root, dirs_, files_ in os.walk(path, followlinks=True):
        for file_ in files_:
            files.append(os.path.join(root, file_))
    return files


with open("./README.md", "r") as f:
    long_description = f.read()

DIR = os.path.abspath(os.path.dirname(__file__))
EXTRA_FILES = get_files_by_path(os.path.join(DIR, "example"))
EXTRA_FILES.extend(get_files_by_path(os.path.join(DIR, "schema")))
EXTRA_FILES.extend(get_files_by_path(os.path.join(DIR, "manifest")))

setup(
    name="esse",
    setup_requires=["setuptools_scm"],
    use_scm_version=True,
    description="Exabyte Source of Schemas and Examples",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Exabyte-io/exabyte-esse",
    author="Exabyte Inc.",
    author_email="info@exabyte.io",
    packages=find_packages(where="src/py", exclude=["src/py/tests*"]),
    package_dir={"": "src/py"},
    package_data={"esse": [f.replace(DIR, "data") for f in EXTRA_FILES]},
    install_requires=[
        "pyyaml>=4.2b1,<6",
        "jsonschema==2.6.0",
        "python-slugify==2.0.1",
        "exabyte_json_include>=2020.10.19",
    ],
    extras_require={
        "test": [
            "coverage[toml]>=5.3",
        ]
    },
    python_requires=">=3.6",
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development",
        "License :: OSI Approved :: Apache Software License",
    ],
)



import os
import yaml

DIR = os.path.dirname(__file__)
SCHEMAS_DIR = os.path.join(DIR, "data/schema")
EXAMPLES_DIR = os.path.join(DIR, "data/example")
MANIFEST_DR = os.path.join(DIR, "data/manifest")

with open(os.path.join(MANIFEST_DR, "properties.yaml")) as f:
    PROPERTIES_MANIFEST = yaml.load(f.read(), Loader=yaml.FullLoader)
    RESULTS = [k for k, v in PROPERTIES_MANIFEST.items() if v.get("isResult")]



import jsonschema

from esse.utils import parseIncludeReferenceStatementsByDir
from esse.settings import SCHEMAS_DIR, EXAMPLES_DIR, PROPERTIES_MANIFEST

SCHEMAS = parseIncludeReferenceStatementsByDir(SCHEMAS_DIR)
EXAMPLES = parseIncludeReferenceStatementsByDir(EXAMPLES_DIR)


class ESSE(object):
    """
    Exabyte Source of Schemas and Examples class.
    """

    def __init__(self):
        self.schemas = SCHEMAS
        self.examples = EXAMPLES

    def get_schema_by_id(self, schemaId):
        return next((s for s in SCHEMAS if s.get("$id") == schemaId), None)

    def validate(self, example, schema):
        """
        Validates a given example against the schema.

        Args:
            example (dict|list): example to validate.
            schema (dict): schema to validate the example with.

        Raises:
            jsonschema.exceptions.ValidationError
        """
        jsonschema.validate(example, schema)

    def get_property_manifest(self, property_):
        """
        Returns the manifest for a given property.

        Args:
            property_ (str): property name.

        Returns:
             dict
        """
        return PROPERTIES_MANIFEST.get(property_, {})
